#include "squidwall.h"

#define HAVE_PATCHED_SQUID  0
#define CLAMD_VERSION_CHECK 10

/* buffers for clamd command connection */
static char clamd_iBUF[BUFSIZE]; buffer clamd_in;
static char clamd_oBUF[BUFSIZE]; buffer clamd_out;

/* buffers for clamd data/stream connection */
static char clams_oBUF[BUFSIZE]; buffer clams_out;

/* buffers for proxy connection */
static char proxy_iBUF[BUFSIZE]; buffer proxy_in;
static char proxy_oBUF[BUFSIZE]; buffer proxy_out;

stralloc virus;
stralloc clamav;

int check_clamd(void)
{
	int64 fd_clamd, fd_stream, fd_proxy;
	static int clamd_check_version=CLAMD_VERSION_CHECK;
	size_t len=0;
	char ch;
	int r;
	stralloc clamdtext;
	stralloc proxytext;

	if (!str_equal(squid_method, "GET")) { return 0; }
	stralloc_init(&clamdtext);
	stralloc_init(&proxytext);

	/* get version of clamav */
	if (clamd_check_version % CLAMD_VERSION_CHECK == 0) {
		fd_clamd=socket_tcp4();
		if (fd_clamd == -1) return panic("socket() for clamd failed");
    		r=socket_connect4(fd_clamd, (char*)&clamd_ip, clamd_port);
		if (r == -1) return panic("connect() to clamd failed");
		if (!io_fd(fd_clamd)) return panic("io_fd() failed");

		buffer_init(&clamd_in,   read, fd_clamd, clamd_iBUF, sizeof clamd_iBUF);
		buffer_init(&clamd_out, write, fd_clamd, clamd_oBUF, sizeof clamd_oBUF);

		io_wantwrite(fd_clamd); io_wait(); io_dontwantwrite(fd_clamd);
		buffer_putsflush(&clamd_out, "VERSION\n");

		io_wantread(fd_clamd); io_wait(); io_dontwantread(fd_clamd);
		if (buffer_getnewline_sa(&clamd_in, &clamav) == 0)
		    return panic("can't read response of clamd");
		clamav.s[clamav.len]=0;

		if (!str_start(clamav.s, "ClamAV "))
			return panic("bad response from clamd, expected: 'ClamAV ' ...");
	
		clamd_in.fd=-1;  buffer_close(&clamd_in);
		clamd_out.fd=-1; buffer_close(&clamd_out);
		io_close(fd_clamd);
		clamd_check_version=0;
	}
	clamd_check_version++;

	/* connection to clamd */
	fd_clamd=socket_tcp4();
	if (fd_clamd == -1) return panic("socket() for clamd failed");
        r=socket_connect4(fd_clamd, (char*)&clamd_ip, clamd_port);
	if (r == -1) return panic("connect() to clamd failed");
	if (!io_fd(fd_clamd)) return panic("io_fd() failed");

	buffer_init(&clamd_in,   read, fd_clamd, clamd_iBUF, sizeof clamd_iBUF);
	buffer_init(&clamd_out, write, fd_clamd, clamd_oBUF, sizeof clamd_oBUF);

	io_wantwrite(fd_clamd); io_wait(); io_dontwantwrite(fd_clamd);
	buffer_putsflush(&clamd_out, "STREAM\n");

	io_wantread(fd_clamd); io_wait(); io_dontwantread(fd_clamd);
	if (buffer_getnewline_sa(&clamd_in, &clamdtext) == 0) return panic("can't read response of clamd");
	clamdtext.s[clamdtext.len]=0;
	if (!str_start(clamdtext.s, "PORT "))
		return panic("bad response from clamd, expected: PORT xxx");

	r=scan_uint(clamdtext.s+5, &clams_port);
	if (r == 0) panic("can't read port of clamd stream");

	/* open connection to clamd stream socket */
	fd_stream=socket_tcp4();
	if (fd_stream == -1) return panic("socket() for clamd failed");
        r=socket_connect4(fd_stream, (char*)&clamd_ip, clams_port);
	if (r == -1) return panic("connect() to clamd failed");
	buffer_init(&clams_out, write, fd_stream, clams_oBUF, sizeof clams_oBUF);
	if (!io_fd(fd_stream)) panic("io_fd() failed");

	/* open connection to proxy */
	fd_proxy=socket_tcp4();
	if (fd_stream == -1) return panic("socket() for proxy failed");
        r=socket_connect4(fd_proxy, (char*)&proxy_ip, proxy_port);
	if (r == -1) return panic("connect() to proxy failed");
	if (!io_fd(fd_proxy)) return panic("io_fd() failed");

	buffer_init(&proxy_in,   read, fd_proxy, proxy_iBUF, sizeof proxy_iBUF);
	buffer_init(&proxy_out, write, fd_proxy, proxy_oBUF, sizeof proxy_oBUF);

	io_wantwrite(fd_proxy); io_wait(); io_dontwantwrite(fd_proxy);
	buffer_puts(&proxy_out, "GET ");
	buffer_puts(&proxy_out, squid_url);
	buffer_puts(&proxy_out, " HTTP/1.0\r\nHost: ");
	buffer_putsa(&proxy_out, &webserver);
	buffer_putsflush(&proxy_out, "\r\n\r\n");

	while (buffer_get(&proxy_in, &ch, 1) == 1) {
		buffer_put(&clams_out, &ch, 1);
		if (++len >= clamd_maxsize) break;
#if HAVE_PATCHED_SQUID
		/* can be used, if squid understands a '*' for ... xxx patch is not ready! */
		if (len % BUFSIZE == 0) buffer_putsflush(buffer_1, "*");
#endif
	}
	buffer_flush(&clams_out);

	/* close proxy and clamd stream connection */
	proxy_out.fd=-1; buffer_close(&proxy_out); io_close(fd_proxy);
	clams_out.fd=-1; buffer_close(&clams_out); io_close(fd_stream);
	stralloc_free(&proxytext);

	/* see, what clamd means to our stream */
	io_wantread(fd_clamd); io_wait(); io_dontwantread(fd_clamd);
	if (!stralloc_copys(&clamdtext, "")) die_nomem();
	while (buffer_getline_sa(&clamd_in, &clamdtext) != 0);
	
	/* close clamd connection */
	clamd_out.fd=-1; buffer_close(&clamd_out); io_close(fd_clamd);


	if (clamdtext.len != 11) { /* "stream: OK\n" -> if len==11 => no virus! */
		/* "stream: Worm.Sober.I FOUND\n" -> "Worm.Sober.I" */
		clamdtext.s[clamdtext.len-7]=0; /* remove " FOUND\n" */
		if (!stralloc_copys(&virus, clamdtext.s+8)) die_nomem();
		stralloc_free(&clamdtext);

		/* log the virus site */
		logs("virus:\t");
		logsa(&url); logs("\t");
		logsa(&ip); logs("\t");
		logsa(&host); logs("\t");
		logsa(&user); logs("\t");
		logsa(&method); logs("\t");
		logsa(&virus); logs("\n");
		logflush();

		redir_site(TYPE_CLAMD);

		return FLAGS_DENY;
	}

	stralloc_free(&clamdtext);
	return 0;
}
