#include "squidwall.h"

/*
 *
 * 1. check if '!' was modified
 * 2. if modified, scan the whole directory to an array
 */

int scan_directory(array *root, const char *path, time_t *mtime)
{
	DIR *D;
	struct dirent *d;
	struct squidwall_acl *acl;
	struct stat s;
	stralloc sa;
	int i, number=0;

	stralloc_init(&sa);
	if (!stralloc_copys(&sa, path)) die_nomem();
	if (!stralloc_cats(&sa, "/!")) die_nomem();
	sa.s[sa.len]=0;

	/* if stat fails, we return ok -> no hosts/users acl directory */
	if (stat(sa.s, &s) == -1) { stralloc_free(&sa); return -1; }
	stralloc_free(&sa);

	/* was not modified... */
	if (*mtime == s.st_mtime) { return 0; }
	*mtime = s.st_mtime;

	D=opendir(path);
	if (!D) return -1;
	while ((d=readdir(D))) {

		/* skip '.' and '..' */
		if (d->d_name[0]=='.' && d->d_name[1]==0) continue;
		if (d->d_name[0]=='.' && d->d_name[1]=='.' && d->d_name[2]==0) continue;

		/* filename is to short */
		if (d->d_reclen <= 1) continue;

		/* filename begins with a false character */
		if (d->d_name[0]!='+' && d->d_name[0]!='-' && d->d_name[0]!='?') continue;

		acl=array_allocate(root, sizeof(struct squidwall_acl), number);
		if (!acl) die_nomem();
		byte_zero(acl, sizeof(struct squidwall_acl));
		stralloc_copys(&acl->name, d->d_name+1); /* copy only the name */
		acl->name.s[acl->name.len]=0;

		/* get names */
		switch (d->d_name[0]) {
			case '+': acl->flags=FLAGS_ALLOW; break;
			case '?': acl->flags=FLAGS_SCAN; break;
			case '-': acl->flags=FLAGS_DENY; break;
		}
		number++;
	}
	closedir(D);
	logflush();

	/* free unused stralloc's of array */
	for (i=number; i < array_length(root, sizeof(struct squidwall_acl)); i++) {
		acl = array_get(root, sizeof(struct squidwall_acl), i);
		logs("free() @ scan_dir()\n");logflush();
		stralloc_free(&acl->name);
	}

	/* destroy unsued array members */
	array_truncate(root, sizeof(struct squidwall_acl), number);

	return 0;
}
