#include "squidwall.h"

void setup(void);

/* fd of logfile */
int64 fd_log;

/* ip's and ports */
struct ip_address clamd_ip;
struct ip_address proxy_ip;
unsigned long clamd_maxsize;
unsigned clamd_port;
unsigned clams_port;
unsigned proxy_port;

/* destination websites for squid */
char *redir_baduser=0;
char *redir_badhost=0;
char *redir_virusfound=0;
char *redir_safesite=0;

/* directories with the host/user files */
char *hosts_dir=0;
char *users_dir=0;

/* pointer to the infos from squid @ fd=0 */
char *squid_url=0;
char *squid_ip=0;
char *squid_host=0;
char *squid_user=0;
char *squid_method=0;

/* decoded infos */
stralloc ip;
stralloc url;
stralloc host;
stralloc user;
stralloc method;
stralloc webserver; /* webserver in the url which comes from squid */
stralloc redirsite; /* contains the side, which will be given to squid (without \n) */

/* read configuration environment variables */
void setup(void)
{
	int r;
	char *x;

	/* init the stralloc's */
	stralloc_init(&ip);
	stralloc_init(&url);
	stralloc_init(&host);
	stralloc_init(&user);
	stralloc_init(&method);
	stralloc_init(&webserver);
	stralloc_init(&redirsite);

	/* open logfile */
	x=env_get("LOG_FILE");
	if (!x) x=LOG_FILE;
	if (!io_appendfile(&fd_log, x)) panic("could not open logfile");

	/* virus warning url */
	redir_virusfound=env_get("REDIR_VIRUSFOUND");
	if (!redir_virusfound) redir_virusfound=REDIR_VIRUSFOUND;

	/* user is not permitted to use the proxy */
	redir_baduser=env_get("REDIR_BADUSER");
	if (!redir_baduser) redir_baduser=REDIR_BADUSER;

	/* host is not permitted to use the proxy */
	redir_badhost=env_get("REDIR_BADHOST");
	if (!redir_badhost) redir_badhost=REDIR_BADHOST;

	/* this url is can't be denied */
	redir_safesite=env_get("REDIR_SAFESITE");
	if (!redir_safesite) redir_safesite=REDIR_SAFESITE;

	/* directory for the userfiles */
	users_dir=env_get("USERS_DIR");
	if (!users_dir) users_dir=USERS_DIR;

	/* directory for the hostfiles */
	hosts_dir=env_get("HOSTS_DIR");
	if (!hosts_dir) hosts_dir=HOSTS_DIR;

	/* read clamd configuration */
	x=env_get("CLAMD");
	if (!x) x=CLAMD;
	r=ip4_scan(x, (char*)&clamd_ip);
	if (!r) panic("can't get ip of $CLAMD!");
	r=scan_uint(x+r+1, &clamd_port);
	if (!r) panic("can't get port of $CLAMD!");
	
	/* read proxy configuration */
	x=env_get("PROXY");
	if (!x) x=PROXY;
	r=ip4_scan(x, (char*)&proxy_ip);
	if (!r) panic("can't get ip of $PROXY!");
	r=scan_uint(x+r+1, &proxy_port);
	if (!r) panic("can't get port of $PROXY!");

	/* read max filesize for clamd virus scans */
	x=env_get("MAX_SIZE");
	if (!x) {
		clamd_maxsize=MAX_SIZE;
	} else {
		r=scan_ulong(x, &clamd_maxsize);
		if (!r) panic("can't read environment $MAX_SIZE!");
	}
	
	return;
}

int main(int argc, char **argv)
{
	unsigned int i;
	stralloc sa;
	int r, flags;

	stralloc_init(&sa);
	setup();

	if (!io_fd(0)) return 111;
	if (!io_fd(1)) return 111;

	for (;;) {
		int state=0;

		/* free old stuff */
		stralloc_copys(&sa, "");
		stralloc_copys(&ip, "");
		stralloc_copys(&url, "");
		stralloc_copys(&host, "");
		stralloc_copys(&user, "");
		stralloc_copys(&method, "");
		stralloc_copys(&webserver, "");
		stralloc_copys(&redirsite, "");

		for (;;) {
			io_wantread(0); io_wait(); io_dontwantread(0);
			r=buffer_getline_sa(buffer_0, &sa);
			if (r == 0) return 0;
			if (r == 1) break;
		}
		sa.s[sa.len-1]=0; /* remove the newline */

		/* format: url ip/host user method
		 * sample: http://www.mcmilk.de/ 192.168.100.10/aspire mcmilk GET
		 * state=0 url=http://www.mcmilk.de/
		 * state=1 ip=192.168.100.10
		 * state=2 host=aspire
		 * state=3 user=root
		 * state=4 method=GET
		 */

		squid_url=sa.s;
		for (i=0; i<=sa.len; i++) {
			switch (state) {
			case 0:
			if (sa.s[i] == ' ') {
				int start, len;
				sa.s[i]=0; state=1; squid_ip=sa.s+i+1;
				/* extract the hostname of the url
				 * http://www.google.de/ -> www.google.de
				 * s1=6 len=13
				 */
				start=str_chr(squid_url, ':');
				start+=3; /* remove '://' */
				len=str_chr(squid_url+start, '/');
				if (!stralloc_copyb(&webserver, squid_url+start, len)) die_nomem();
				webserver.s[webserver.len]=0;
			}
			break;
			case 1:
			if (sa.s[i] == '/') {
				sa.s[i]=0; state=2; squid_host=sa.s+i+1;
			}
			break;
			case 2:
			if (sa.s[i] == ' ') {
				sa.s[i]=0; state=3; squid_user=sa.s+i+1;
			}
			break;
			case 3:
			if (sa.s[i] == ' ') {
				sa.s[i]=0; state=4; squid_method=sa.s+i+1;
			}
			break;
			} /* switch() */
		}

		if (state != 4) {
			logs("error: incorrect input from squid!\n");
			buffer_putsaflush(buffer_1, &sa); /* return it to stdout */
			continue;                         /* and ignore it! */
		}
		
		if (str_start(squid_url, redir_safesite)) { goto out; }

		/* squid gives us urlencoded things - we decode them first! */
		if (!scan_to_sa(scan_urlencoded,  squid_ip, &ip)) die_nomem();
		if (!scan_to_sa(scan_urlencoded2, squid_url, &url)) die_nomem();
		if (!scan_to_sa(scan_urlencoded,  squid_host, &host)) die_nomem();
		if (!scan_to_sa(scan_urlencoded2, squid_user, &user)) die_nomem();
		if (!scan_to_sa(scan_urlencoded,  squid_method, &method)) die_nomem();
		ip.s[ip.len]=0;
		url.s[url.len]=0;
		host.s[host.len]=0;
		user.s[user.len]=0;
		method.s[method.len]=0;

		/* we check in this order:
		 *
		 * 1. is the user okay ?
		 * 2. is the host allowed ?
		 * 3. is there a virus in the website ?
		 */

		flags=0;
		switch (check_acl(TYPE_USER)) {
			case FLAGS_SCAN:
				set(flags, FLAGS_SCAN);
				break;
			case FLAGS_DENY:
				redir_site(TYPE_USER); goto out;
				break;
		}

		switch (check_acl(TYPE_HOST)) {
			case FLAGS_SCAN:
				set(flags, FLAGS_SCAN);
				break;
			case FLAGS_DENY:
				redir_site(TYPE_HOST); goto out;
				break;
		}

		/* scan with clamd */
		if (isset(flags, FLAGS_SCAN)) {
			check_clamd();
		}

		out:
		/* give squid an empty newline '\n' or the whole redirsite */
		io_wantwrite(1); io_wait(); io_dontwantwrite(1);
		buffer_putsa(buffer_1, &redirsite);
		buffer_puts(buffer_1, "\n");
		while (buffer_flush(buffer_1) == -1);
	}

	return 0;
}
