#ifndef SW_SQUIDWALL_H
#define SW_SQUIDWALL_H

#include "params.h"

#define VERSION    "squidwall 0.1"
#define BUFSIZE    9000

/* system */
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/poll.h>
#include <sys/time.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <dirent.h>
#include <fcntl.h>
#include <errno.h>
#include <wait.h>
#include <time.h>

/* libowfat */
#include "array.h"
#include "buffer.h"
#include "case.h"
#include "env.h"
#include "fmt.h"
#include "ip4.h"
#include "ip6.h"
#include "io.h"
#include "scan.h"
#include "socket.h"
#include "stralloc.h"
#include "textcode.h"

struct ip_address { unsigned char d[4]; };

#define FLAGS_ALLOW  0
#define FLAGS_SCAN   1
#define FLAGS_DENY   2

/* you may change this flag to FLAGS_DENY or so... */
#define FLAGS_DEFAULT FLAGS_ALLOW
struct squidwall_acl {
	stralloc name;
	int flags;
};

/* fd of logfile */
extern int64 fd_log;

/* ip's and ports */
extern struct ip_address clamd_ip;
extern struct ip_address proxy_ip;
extern unsigned long clamd_maxsize;
extern unsigned clamd_port;
extern unsigned clams_port;
extern unsigned proxy_port;

/* pointer to the infos from squid @ fd=0 */
extern char *squid_url;
extern char *squid_ip;
extern char *squid_host;
extern char *squid_user;
extern char *squid_method;

/* destination websites for squid */
extern char *redir_baduser;
extern char *redir_badhost;
extern char *redir_virusfound;

/* directories with the host/user files */
extern char *hosts_dir;
extern char *users_dir;

/* decoded infos */
extern stralloc ip;
extern stralloc url;
extern stralloc host;
extern stralloc user;
extern stralloc method;
extern stralloc webserver;
extern stralloc redirsite;

/* panic.c */
extern int panic(char *message);
//#define die_nomem() panic("nomem() @ file:" __FILE__ "!")
#define die_nomem() panic("nomem")

/* log.c */
extern void logs(char *message);
extern void logsa(stralloc *sa);
extern void logflush(void);

/* check_acl.c */
extern int check_acl(int type);

/* check_clamd.c */
extern int check_clamd(void);

/* redir_site.c */
#define TYPE_USER  0
#define TYPE_HOST  1
#define TYPE_CLAMD 2
extern void redir_site(int type);

/* scan_directory.c */
extern int scan_directory(array *root, const char *path, time_t *mtime);

#define isset(v,x)     (((v)&(x)) == (x))
#define set(v,x)       (v) |=  (x)
#define unset(v,x)     (v) &= ~(x)

#endif
